using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO.Ports;
using System.Diagnostics;

namespace HALibs.VAMS0808.Test.WinForms {
	public partial class Form1 : Form {
		Connection conn;
		bool updatingMode;
		bool updatingLocked;

		public Form1() {
			InitializeComponent();
			Trace.Listeners.Add(new ConsoleTraceListener());
		}

		private void Form1_Load(object sender, EventArgs e) {
			conn = new Connection(new SerialPort("/dev/ttyS0", 9600, Parity.None, 8, StopBits.One));
			conn.State.AllChannelsSelectedChanged += new MatrixState.BooleanFieldModifiedEventHandler(State_AllChannelsSelectedChanged);
			conn.State.CurrentChannelIndexChanged += new MatrixState.IntegerFieldModifiedEventHandler(State_CurrentChannelIndexChanged);
			conn.State.AudioChannels.ChannelModified += new ChannelCollection.ChannelModifiedEventHandler(AudioChannels_ChannelModified);
			conn.State.VideoChannels.ChannelModified += new ChannelCollection.ChannelModifiedEventHandler(VideoChannels_ChannelModified);
			conn.State.SwitchModeChanged += new MatrixState.SwitchModeModifiedEventHandler(State_SwitchModeChanged);
			conn.State.LockedChanged += new MatrixState.BooleanFieldModifiedEventHandler(State_LockedChanged);

			conn.RequestStateUpdate();
		}

		void State_LockedChanged(bool previousValue, bool newValue) {
			updatingLocked = true;
			chkLocked.Checked = newValue;
			updatingLocked = false;
		}

		void State_SwitchModeChanged(SwitchMode previousValue, SwitchMode newValue) {
			updatingMode = true;
			if (newValue == SwitchMode.AudioVideo)
				rdbAV.Checked = true;
			else if (newValue == SwitchMode.Audio)
				rdbAudio.Checked = true;
			else
				rdbVideo.Checked = true;

			updatingMode = false;
		}

		void VideoChannels_ChannelModified(int channel, int previousValue, int newValue) {
			lblVideoChannels.Text = conn.State.VideoChannels.ToString();
		}

		void AudioChannels_ChannelModified(int channel, int previousValue, int newValue) {
			lblAudioChannels.Text = conn.State.AudioChannels.ToString();
		}

		void State_CurrentChannelIndexChanged(int previousValue, int newValue) {
			//throw new Exception("The method or operation is not implemented.");
		}

		void State_AllChannelsSelectedChanged(bool previousValue, bool newValue) {
			//throw new Exception("The method or operation is not implemented.");
		}

		private void inputChannelButton_Click(object sender, EventArgs e) {
			conn.SelectInputChannel(int.Parse(((Button)sender).Text));
		}

		private void outputChannelButton_Click(object sender, EventArgs e) {
			conn.SelectOutputChannel(int.Parse(((Button)sender).Text));
		}

		private void btnAll_Click(object sender, EventArgs e) {
			conn.SelectAllOutputChannels();
		}

		private void btnOff_Click(object sender, EventArgs e) {
			conn.DisableCurrentChannel();
		}

		private void btnAV_Click(object sender, EventArgs e) {
			
		}

		private void btnAudio_Click(object sender, EventArgs e) {
			
		}

		private void btnVideo_Click(object sender, EventArgs e) {
			
		}

		private void btnUpdate_Click(object sender, EventArgs e) {
			conn.RequestStateUpdate();
		}

		private void lblInputs_Click(object sender, EventArgs e) {

		}

		private void rdbAV_CheckedChanged(object sender, EventArgs e) {
			if (rdbAV.Checked && !updatingMode)
				conn.SetSwitchMode(SwitchMode.AudioVideo);
		}

		private void rdbVideo_CheckedChanged(object sender, EventArgs e) {
			if (rdbVideo.Checked && !updatingMode)
				conn.SetSwitchMode(SwitchMode.Video);
		}

		private void rdbAudio_CheckedChanged(object sender, EventArgs e) {
			if (rdbAudio.Checked && !updatingMode)
				conn.SetSwitchMode(SwitchMode.Audio);
		}

		private void chkLocked_CheckedChanged(object sender, EventArgs e) {
			if (!updatingLocked) {
				if (chkLocked.Checked)
					conn.Lock();
				else
					conn.UnLock();
			}
		}
	}
}