// Unofficial VAMS-0808 RS232 interface wrapper
// Originally written by Simon Smith (opensource-ks@killswtch.net)
// http://www.killswtch.net/

using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;

namespace HALibs.VAMS0808 {
	/// <summary>
	/// A collection for monitoring the state of channels.
	/// </summary>
	public class ChannelCollection : IList<int> {
		public delegate void ChannelModifiedEventHandler(int channel, int previousValue, int newValue);

		int[] channels = new int[8];

		public event ChannelModifiedEventHandler ChannelModified;

		#region IList<int> Members

		public int IndexOf(int item) {
			for (int i = 0; i < 8; i++)
				if (this[i] == item) return i;
			return -1;
		}

		public void Insert(int index, int item) {
			throw new Exception("Channels cannot be inserted.");
		}

		public void RemoveAt(int index) {
			throw new Exception("Channels cannot be removed.");
		}

		public int this[int index] {
			get {
				if (index > Count - 1)
					throw new ArgumentOutOfRangeException();
				return channels[index];
			}
			set {
				int oldValue = channels[index];
				channels[index] = value;

				if (oldValue != value && ChannelModified != null) {
					ChannelModified(index + 1, oldValue, value);
					Trace.WriteLine(String.Format("Channel {0} changed from {1} to {2}", index + 1, oldValue, value));
				}
			}
		}

		#endregion

		#region ICollection<OutputChannel> Members

		public void Add(int item) {
			throw new Exception("Channels cannot be added.");
		}

		public void Clear() {
			throw new Exception("Channels cannot be removed.");
		}

		public bool Contains(int item) {
			return IndexOf(item) > -1;
		}

		public void CopyTo(int[] array, int arrayIndex) {
			for (int i = arrayIndex; i - arrayIndex < 8; i++) {
				array[i] = channels[i - arrayIndex];
			}
		}

		public int Count {
			get { return channels.Length; }
		}

		public bool IsReadOnly {
			get { return false; }
		}

		public bool Remove(int item) {
			throw new Exception("Channels cannot be removed.");
		}

		#endregion

		#region IEnumerable<int> Members

		public IEnumerator<int> GetEnumerator() {
			return new ChannelEnumerator(this);
		}

		#endregion

		#region IEnumerable Members

		System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator() {
			throw new Exception("The method or operation is not implemented.");
		}

		#endregion

		public override string ToString() {
			StringBuilder sb = new StringBuilder(Count);

			foreach (int i in this) {
				sb.Append(i.ToString());
			}

			return sb.ToString();
		}
	}
}
